﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using Infragistics.Win;
using Infragistics.Win.UltraWinSchedule.DayView;
using Infragistics.Win.UltraWinSchedule;
using System.Drawing.Drawing2D;
using Infragistics.Win.UltraWinEditors;

namespace WS_Color
{
    public partial class Form1 : Form
    {
        private MyDrawFilter drawFilter;

        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            ultraCalendarInfo1.SelectedDateRanges.Clear();
            ultraCalendarInfo1.SelectedDateRanges.Add(DateTime.Today, DateTime.Today.AddDays(7));
            drawFilter = new MyDrawFilter();
            ultraDayView1.DrawFilter = drawFilter;
            ultraColorPicker2.Visible = ultraCheckEditor1.Checked;
            ultraLabel9.Visible = ultraCheckEditor1.Checked;
        }

        private void btnSet_Click(object sender, EventArgs e)
        {
            var backColorData = new BackColorData();
            backColorData.StartDate = DateTime.Parse(udeStart.Value.ToString());
            backColorData.EndDate = DateTime.Parse(udeEnd.Value.ToString());
            backColorData.StartHour = DateTime.Parse(udeHourStart.Value.ToString());
            backColorData.EndHour = DateTime.Parse(udeHourEnd.Value.ToString());
            backColorData.Color = ultraColorPicker1.Color;
            backColorData.Color2 = ultraColorPicker2.Color;
            backColorData.UseGradient = ultraCheckEditor1.Checked;
            drawFilter.BackColorData = backColorData;
            ultraDayView1.Invalidate();
        }

        private void ultraCheckEditor1_CheckedChanged(object sender, EventArgs e)
        {
            ultraColorPicker2.Visible = ultraCheckEditor1.Checked;
            ultraLabel9.Visible = ultraCheckEditor1.Checked;
        }

        private void udeHour_EditorSpinButtonClick(object sender, Infragistics.Win.UltraWinEditors.SpinButtonClickEventArgs e)
        {
            var editor = (UltraDateTimeEditor)e.Context;
            if (e.ButtonType == Infragistics.Win.UltraWinEditors.SpinButtonItem.NextItem)
            {
                editor.Value = DateTime.Parse(editor.Value.ToString()).AddHours(1);
            }
            if (e.ButtonType == Infragistics.Win.UltraWinEditors.SpinButtonItem.PreviousItem)
            {
                editor.Value = DateTime.Parse(editor.Value.ToString()).AddHours(-1);
            }
        }
    }

    public class MyDrawFilter : IUIElementDrawFilter
    {
        private BackColorData backColorData;

        public MyDrawFilter()
        {
        }

        public BackColorData BackColorData
        {
            get { return backColorData; }
            set { backColorData = value; }
        }

        #region IUIElementDrawFilter Members

        public bool DrawElement(DrawPhase drawPhase, ref UIElementDrawParams drawParams)
        {
            if (drawParams.Element is TimeSlotUIElement)
            {
                if (backColorData != null)
                {
                    // If you wanted, you could put additional tests in here to see if the TimeSlot is within the
                    // working hours.  You could also change the single BackColorData property to a collection and
                    // have different background colors for different ranges.
                    var context = drawParams.Element.GetContext() as TimeSlot;
                    var element = drawParams.Element as TimeSlotUIElement;
                    DateTime date = new DateTime(element.Date.Year, element.Date.Month, element.Date.Day, 
                        context.StartTime.Hour, context.StartTime.Minute, context.StartTime.Second);
                    if ((date >= backColorData.StartDate) && (date < backColorData.EndDate) 
                        && (date.Hour >= backColorData.StartHour.Hour) && (date.Hour < backColorData.EndHour.Hour) )
                    {
                        var g = drawParams.Graphics;
                        if (backColorData.UseGradient)
                        {
                            //this creates a new gradient brush from top to bottom
                            using (var brush = new LinearGradientBrush(new Point(element.Rect.Left, element.Rect.Top), 
                                new Point(element.Rect.Left, element.Rect.Bottom), backColorData.Color, backColorData.Color2))
                            {
                                g.FillRectangle(brush , element.Rect);
                            }
                        }
                        else
                        {
                            using (var brush = new SolidBrush(backColorData.Color))
                            {
                                g.FillRectangle(brush, element.Rect);
                            }
                        }
                        return true;
                    }
                }
            }
            return false;
        }

        public DrawPhase GetPhasesToFilter(ref UIElementDrawParams drawParams)
        {
            return DrawPhase.BeforeDrawElement;
        }

        #endregion
    }

    public class BackColorData
    {
        public DateTime StartDate { get; set; }
        public DateTime EndDate { get; set; }
        public DateTime StartHour { get; set; }
        public DateTime EndHour { get; set; }
        public Color Color { get; set; }
        public Color Color2 { get; set; }
        public bool UseGradient { get; set; }
    }
}
